"""
此模块提供事件总线相关类。
"""

import asyncio
from collections import defaultdict
from typing import Any, Callable, List


class EventBus:
    def __init__(self):
        self._subscribers = defaultdict(set)

    def subscribe(self, event: str, func: Callable) -> None:
        self._subscribers[event].add(func)

    def unsubscribe(self, event: str, func: Callable) -> None:
        if func in self._subscribers[event]:
            self._subscribers[event].remove(func)

    def on(self, event: str) -> Callable:
        def decorator(func: Callable) -> Callable:
            self.subscribe(event, func)
            return func

        return decorator

    async def emit(self, event: str, *args, **kwargs) -> List[Any]:
        results = []
        coros = []
        for f in self._subscribers[event]:
            coros.append(f(*args, **kwargs))
        if coros:
            results += await asyncio.gather(*coros)
        return results



from quart import Quart, Response, abort, jsonify, request

from . import Receiver


class ReportReceiver(Receiver):
    def __init__(self, app: Quart, endpoint: str = '/mirai'):
        super().__init__()
        app.add_url_rule(path=endpoint,
                         endpoint=endpoint,
                         view_func=self._postreceive,
                         methods=['POST'])

    async def _postreceive(self) -> Response:
        payload = await request.json
        if not isinstance(payload, dict):
            abort(400)

        response = await self._handle_event(payload)
        if isinstance(response, dict):
            return jsonify(response)
        return Response('', 204)



"""
此模块提供了工具函数。
"""

import re


def camelCase(name: str) -> str:
    """
    将下划线命名法(snake_case)转换为小驼峰式命名法(camelCase)。
    """
    return re.sub(r'(.*?)_([a-z])', lambda m: m.group(1) + m.group(2).upper(),
                  name)


def snake_case(name: str) -> str:
    """
    将小驼峰式命名法(camelCase)转换为下划线命名法(snake_case)。
    """
    return re.sub(r'(.*?)([A-Z])',
                  lambda m: m.group(1) + '_' + m.group(2).lower(), name)



"""
此模块提供了 Mirai API HTTP 的事件相关类。
"""

from enum import Enum
from typing import Any, Dict, Optional, Union

from .message import MessageChain
from .utils import snake_case

__all__ = ['Event']


class Event(dict):
    @property
    def type(self) -> str:
        return self['type']

    def __repr__(self) -> str:
        return f'<{self.__class__.__name__}, {super().__repr__()}>'


def from_payload(payload: Dict[str, Any]) -> Optional[Event]:
    """
    从 Mirai API HTTP 事件数据构造 `Event` 对象。
    """
    def _parse(data):
        if isinstance(data, dict):
            return {
                snake_case(k):
                MessageChain(v) if k == 'messageChain' else _parse(v)
                for k, v in data.items()
            }
        elif isinstance(data, list):
            return [_parse(x) for x in data]
        else:
            return data

    try:
        e = Event(_parse(payload))
        _ = e.type
        return e
    except KeyError:
        return None



"""
此模块提供了消息相关类。
参考了https://github.com/cqmoe/python-aiocqhttp/blob/master/aiocqhttp/message.py
"""
import re
from typing import Any, Dict, Iterable, Optional, Tuple, Union

from .utils import camelCase

__all__ = [
    'MessageSegment', 'MessageChain', 'At', 'AtAll', 'Face', 'Plain', 'Image',
    'FlashImage', 'Xml', 'Json', 'App', 'Poke'
]


class MessageSegment(dict):
    """
    消息段，即表示成字典的 Mirai 消息对象。
    不建议手动构造消息段；建议使用此类的静态方法构造，例如：
    ```py
    at_seg = MessageSegment.at(target=10001000)
    ```
    可进行判等和加法操作，例如：
    ```py
    assert at_seg == MessageSegment.at(target=10001000)
    msg: MessageChain = at_seg + MessageSegment.face(face_id=14)
    ```
    """
    def __init__(self,
                 d: Optional[Union[str, Dict[str, Any]]] = None,
                 *,
                 type: Optional[str] = None,
                 **extra):
        super().__init__()
        if isinstance(d, str):
            self['type'] = 'Plain'
            self['text'] = d
        elif isinstance(d, dict) and d.get('type'):
            self.update(d)
        elif type:
            self.type = type
            self.update(extra)
        else:
            raise ValueError('the \'type\' field cannot be None or empty')

    @property
    def type(self):
        return self['type']

    @type.setter
    def type(self, type_):
        self['type'] = type_

    def __str__(self):
        if self.type == 'Plain':
            return self['text']
        params = ','.join(f'{k}={str(self[k])}'
                          for k in self.keys() - {'type'})
        if params:
            params = '::' + params
        return '[{}{}]'.format(self.type, params)

    def __repr__(self):
        return self.__str__()

    def __eq__(self, other):
        if not isinstance(other, MessageSegment):
            return False
        return self.type == other.type and super().__eq__(other)

    def __add__(self, other: Any):
        return MessageChain(self).__add__(other)


def At(target: int) -> MessageSegment:
    return MessageSegment(type='At', target=target)


def AtAll() -> MessageSegment:
    return MessageSegment(type='AtAll')


def Face(face: Union[int, str]) -> MessageSegment:
    if isinstance(face, int):
        return MessageSegment(type='Face', face_id=face)
    else:
        return MessageSegment(type='Face', name=face)


def Plain(text: str) -> MessageSegment:
    return MessageSegment(type='Plain', text=text)


_SCHEMES = ('http://', 'https://', 'ftp://')


def Image(image: str) -> MessageSegment:
    if re.match(
            r'\{[A-F0-9]{8}-[A-F0-9]{4}-[A-F0-9]{4}-[A-F0-9]{4}-[A-F0-9]{12}\}.mirai',
            image
    ) or re.match(
            r'/[a-f0-9]{8}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{12}',
            image):
        return MessageSegment(type='Image', image_id=image)
    elif image.startswith(_SCHEMES):
        return MessageSegment(type='Image', url=image)
    else:
        return MessageSegment(type='Image', path=image)


def FlashImage(image: str) -> MessageSegment:
    if re.match(
            r'\{[A-F0-9]{8}-[A-F0-9]{4}-[A-F0-9]{4}-[A-F0-9]{4}-[A-F0-9]{12}\}.png',
            image
    ) or re.match(
            r'/[a-f0-9]{8}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{12}',
            image):
        return MessageSegment(type='FlashImage', image_id=image)
    elif image.startswith(_SCHEMES):
        return MessageSegment(type='FlashImage', url=image)
    else:
        return MessageSegment(type='FlashImage', path=image)


def Xml(xml: str) -> MessageSegment:
    return MessageSegment(type='Xml', xml=xml)


def Json(json: str) -> MessageSegment:
    return MessageSegment(type='Json', json=json)


def App(content: str) -> MessageSegment:
    return MessageSegment(type='App', content=content)


def Poke(name: str) -> MessageSegment:
    return MessageSegment(type='Poke', name=name)


class MessageChain(list):
    """
    消息，即消息段列表。
    """
    def __init__(self, msg: Any = None, *args, **kwargs):
        """``msg`` 参数为要转换为 `MessageChain` 对象的字符串、列表或字典。"""
        super().__init__(*args, **kwargs)
        try:
            if isinstance(msg, list):
                self.extend(msg)
            elif isinstance(msg, (dict, str)):
                self.append(msg)
        except ValueError:
            raise ValueError('the msg argument is not recognizable')

    def __str__(self):
        return ''.join(map(str, self))

    def __repr__(self):
        return self.__str__()

    def __add__(self, other: Any):
        result = MessageChain(self)
        try:
            if isinstance(other, MessageChain):
                result.extend(other)
            elif isinstance(other, MessageSegment):
                result.append(other)
            elif isinstance(other, list):
                result.extend(map(MessageSegment, other))
            elif isinstance(other, (dict, str)):
                result.append(MessageSegment(other))
            return result
        except ValueError:
            raise ValueError('the addend is not a valid message')

    def append(self, obj: Any) -> Any:
        """在消息末尾追加消息段。"""
        try:
            if isinstance(obj, MessageSegment):
                if self and self[-1].type == 'Plain' and obj.type == 'Plain':
                    self[-1]['text'] += obj['text']
                elif obj.type != 'Plain' or obj['text'] or not self:
                    super().append(obj)
            else:
                self.append(MessageSegment(obj))
            return self
        except ValueError:
            raise ValueError('the object is not a valid message segment')

    def extend(self, msg: Any) -> Any:
        """在消息末尾追加消息（消息段列表）。"""
        try:
            for seg in msg:
                self.append(seg)
            return self
        except ValueError:
            raise ValueError('the object is not a valid message')

    def reduce(self) -> None:
        """
        化简消息，即去除多余消息段、合并相邻纯文本消息段。
        由于 `MessageChain` 类基于 `list`，此方法时间复杂度为 O(n)。
        """
        idx = 0
        while idx < len(self):
            if idx > 0 and self[idx -
                                1].type == 'Plain' and self[idx].type == 'Plain':
                self[idx - 1]['text'] += self[idx]['text']
                del self[idx]
            else:
                idx += 1

    def extract_plain_text(self, reduce: bool = False) -> str:
        """
        提取消息中的所有纯文本消息段，合并，中间用空格分隔。
        ``reduce`` 参数控制是否在提取之前化简消息。
        """
        if reduce:
            self.reduce()

        result = ''
        for seg in self:
            if seg.type == 'Plain':
                result += ' ' + seg['text']
        if result:
            result = result[1:]
        return result



"""
此模块提供了 Mirai API HTTP 的 API 调用相关类。
"""

from functools import partial
from typing import IO, Any, Awaitable, Callable, Dict, Optional

import httpx
from httpx._models import RequestData, RequestFiles, QueryParamTypes

from .exception import *
from .logger import Api as Logger
from .utils import camelCase, snake_case

__all__ = ['Api', 'SessionApi']


class Api:
    def __init__(self, api_root: str):
        self.api_root = api_root.strip('/') + '/' if api_root else None

    async def call_action(self,
                          action: str,
                          method: Optional[str] = 'POST',
                          format: Optional[str] = 'json',
                          *,
                          data: Optional[RequestData] = None,
                          files: Optional[RequestFiles] = None,
                          json: Optional[Any] = None,
                          params: Optional[QueryParamTypes] = None,
                          **kwargs) -> Any:

        if method == 'GET':
            format = 'params'

        args = locals().copy()
        del args['self']
        del args['action']
        args = {k: v for k, v in args.items() if v}
        Logger.info('Calling %s with params: %s', action, repr(args))

        mix = {'data': data, 'files': files, 'json': json, 'params': params}

        def _parse(data, func):
            if isinstance(data, dict):
                return {func(k): _parse(v, camelCase) for k, v in data.items()}
            elif isinstance(data, list):
                return [_parse(x, func) for x in data]
            else:
                return data

        mix = _parse(mix, camelCase)
        kwargs = _parse(kwargs, camelCase)

        mix[format] = mix[format] or {}
        mix[format].update(kwargs)

        url = self.api_root + camelCase(action)

        try:
            async with httpx.AsyncClient() as client:
                resp = await client.request(method, url, **mix)
            try:
                res = resp.json()
            except Exception as e:
                if not 200 <= resp.status_code < 300:
                    raise HttpFailed(resp.status_code)
                raise e
            res = _parse(res, snake_case)
            if not isinstance(res, dict):
                return res
            code = res.get('code')
            if code == 0 or code is None:
                return res
            msg = res.get('msg')
            raise ActionFailed(code, msg)
        except httpx.InvalidURL:
            raise NetworkError('API root url invalid')
        except httpx.HTTPError:
            raise NetworkError('HTTP request failed')

    def __getattr__(self, item: str) -> Callable[..., Awaitable[Any]]:
        """获取一个可调用对象，用于调用对应 API。"""
        if item.startswith(('get', 'fetch', 'peek')):
            return partial(self.call_action, item.split('get_', 1)[-1], 'GET')
        return partial(self.call_action, item)


class SessionApi(Api):
    def __init__(self, api_root: str, auth_key: str, qq: int):
        super().__init__(api_root)
        self.auth_key = auth_key
        self.qq = qq
        self.session_key = None

    async def call_action(self,
                          action: str,
                          method: Optional[str] = 'POST',
                          format: Optional[str] = 'json',
                          **params) -> Any:
        if not self.session_key:
            raise Unauthenticated
        try:
            return await super().call_action(action,
                                             method,
                                             format,
                                             **params,
                                             session_key=self.session_key)
        except ActionFailed as _e:
            switcher = {
                1: InvalidAuthKey,
                2: InvalidBot,
                3: InvalidSession,
                4: Unverified
            }
            e = switcher.get(_e.code)
            if not e:
                raise _e
            raise e(_e.code, _e.msg)

    async def upload_image(self, type: str, img: IO) -> Dict[str, Any]:
        return await self.call_action('upload_image',
                                      format='data',
                                      data={'type': type},
                                      files={'img': img})

    async def auth(self) -> Dict[str, Any]:
        r = await Api.call_action(self, 'auth', auth_key=self.auth_key)
        self.session_key = r.get('session')
        return r

    async def verify(self) -> Dict[str, Any]:
        return await self.call_action('verify', qq=self.qq)

    async def release(self) -> Dict[str, Any]:
        try:
            return await self.call_action('release', qq=self.qq)
        finally:
            self.session_key = None

    async def resp_new_friend(self,
                              *,
                              event_id: int,
                              from_id: int,
                              group_id: int,
                              operate: Optional[int] = 0,
                              message: Optional[str] = "") -> Dict[str, Any]:
        return await self.call_action('resp/new_friend_request_event',
                                      event_id=event_id,
                                      from_id=from_id,
                                      group_id=group_id,
                                      operate=operate,
                                      message=message)

    async def resp_member_join(self,
                               *,
                               event_id: int,
                               from_id: int,
                               group_id: int,
                               operate: Optional[int] = 0,
                               message: Optional[str] = "") -> Dict[str, Any]:
        return await self.call_action('resp/member_join_request_event',
                                      event_id=event_id,
                                      from_id=from_id,
                                      group_id=group_id,
                                      operate=operate,
                                      message=message)

    async def __aenter__(self) -> 'SessionApi':
        await self.auth()
        await self.verify()
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.release()



from typing import IO, Any, Awaitable, Dict, List, Optional, Union

from .message import MessageChain

class Api:
    """
    API 基类。
    实现通过 HTTP 调用 Mirai API HTTP。

    Args:
        api_root: Mirai API HTTP 地址
    """
    api_root: str

    def __init__(self, api_root: str):
        ...

    async def call_action(
            self,
            action: str,
            method: Optional[str] = 'POST',
            format: Optional[str] = 'json',
            **params
        ) -> Any:
        """
        调用 Mirai API HTTP。

        Args:
            action: 所要调用的 API 名。
            method: 调用 API 所使用的 http method。
            format: 上报格式（可选的值有：data / json / params）
            params: API 所需参数

        Returns:
            Mirai API HTTP 返回的结果。
        """
        ...

    async def get_about(
            self
        ) -> Dict[str, Any]:
        """
        获取插件的信息，如版本号。
        """
        ...


class SessionApi(Api):
    """
    会话相关 API 实现类。

    Args:
        api_root: Mirai API HTTP 地址
        auth_key: 创建 Mirai Http Server 时生成的 key，可在启动时指定或随机生成
        qq: Session 将要绑定的 Bot 的 qq 号
    """
    auth_key: str
    qq: int
    session_key: Optional[str]

    def __init__(
            self,
            api_root: str,
            auth_key: str,
            qq: int
        ) -> None:
        super().__init__(api_root)
        ...

    async def auth(
            self
        ) -> Dict[str, Any]:
        """
        验证身份，开始与 Mirai API 的会话。
        根据实现类的不同，此函数可能是异步也可能是同步函数。
        """
        ...

    async def verify(
            self
        ) -> Dict[str, Any]:
        """
        校验并激活 Session，同时将 Session 与 Bot 绑定
        """
        ...

    async def release(
            self
        ) -> Dict[str, Any]:
        """
        释放 session 及其相关资源
        """
        ...

    async def send_friend_message(
            self, *,
            target: int,
            quote: Optional[int] = None,
            message_chain: MessageChain
        ) -> Dict[str, Any]:
        """
        发送好友消息

        Args:
            target: 目标好友的 QQ 号
            quote: 引用消息的 messageId
            message_chain: 消息链
        """
        ...

    async def send_temp_message(
            self, *,
            qq: int,
            group: int,
            quote: Optional[int] = None,
            message_chain: MessageChain
        ) -> Dict[str, Any]:
        """
        发送临时会话消息

        Args:
            qq: 临时会话对象QQ号
            group: 临时会话群号
            quote: 引用消息的 messageId
            message_chain: 消息链
        """
        ...

    async def send_group_message(
            self, *,
            target: int,
            quote: Optional[int] = None,
            message_chain: MessageChain
        ) -> Dict[str, Any]:
        """
        发送群消息

        Args:
            target: 目标群的群号
            quote: 引用消息的 messageId
            message_chain: 消息链
        """
        ...

    async def send_image_message(
            self, *,
            urls: List[str],
            target: Optional[int],
            qq: Optional[int],
            group: Optional[int]
        ) -> List[str]:
        """
        发送图片消息（通过URL）
        除非需要通过此手段获取imageId，否则不推荐使用该接口

        Args:
            urls: 由 url 字符串构成的数组
            target: 发送对象的 QQ 号或群号，可能存在歧义
            qq: 发送对象的 QQ 号
            group: 发送对象的群号
        """
        ...

    async def upload_image(
            self, *,
            type: str,
            img: IO
        ) -> Dict[str, Any]:
        """
        上传图片文件

        Args:
            type: 类型（"friend" 或 "group"）
            img: 图片文件
        """
        ...

    async def recall(
            self, *,
            target: int
        ) -> Dict[str, Any]:
        """
        撤回消息

        Args:
            target: 要撤回的消息的 messageId
        """
        ...

    async def fetch_message(
            self, *,
            count: int
        ) -> Dict[str, Any]:
        """
        获取接收到的最老消息和最老各类事件（会从 MiraiApiHttp 消息记录中删除）

        Args:
            count: 获取消息和事件的数量
        """
        ...

    async def fetch_latest_message(
            self, *,
            count: int
        ) -> Dict[str, Any]:
        """
        获取接收到的最新消息和最新各类事件（会从 MiraiApiHttp 消息记录中删除）

        Args:
            count: 获取消息和事件的数量
        """
        ...

    async def peek_message(
            self, *,
            count: int
        ) -> Dict[str, Any]:
        """
        获取接收到的最老消息和最老各类事件（不会从 MiraiApiHttp 消息记录中删除）

        Args:
            count: 获取消息和事件的数量
        """
        ...

    async def peek_latest_message(
            self, *,
            count: int
        ) -> Dict[str, Any]:
        """
        获取接收到的最新消息和最新各类事件（不会从 MiraiApiHttp 消息记录中删除）

        Args:
            count: 获取消息和事件的数量
        """
        ...

    async def get_message_from_id(
            self, *,
            id: int
        ) -> Dict[str, Any]:
        """
        通过 messageId 获取一条被缓存的消息

        Args:
            id: 获取消息的 messageId
        """
        ...

    async def get_count_message(
            self
        ) -> Union[Awaitable[int], int]:
        """
        查看缓存的消息总数
        """
        ...

    async def get_friend_list(
            self
        ) -> List[Dict[str, Any]]:
        """
        获取好友列表
        """
        ...

    async def get_group_list(
            self
        ) -> List[Dict[str, Any]]:
        """
        获取群列表
        """
        ...

    async def get_member_list(
            self, *,
            target: int
        ) -> List[Dict[str, Any]]:
        """
        获取指定群的群成员列表

        Args:
            target: 指定群的群号
        """
        ...

    async def mute_all(
            self, *,
            target: int
        ) -> Dict[str, Any]:
        """
        指定群全体禁言（需要有相关限权）

        Args:
            target: 指定群的群号
        """
        ...

    async def unmute_all(
            self, *,
            target: int
        ) -> Dict[str, Any]:
        """
        指定群解除全体禁言（需要有相关限权）

        Args:
            target: 指定群的群号
        """
        ...

    async def mute(
            self, *,
            target: int,
            member_id: int,
            time: Optional[int]
        ) -> Dict[str, Any]:
        """
        指定群禁言指定群员（需要有相关限权）

        Args:
            target: 指定群的群号
            member_id: 指定群员QQ号
            time: 禁言时长，单位为秒，最多30天，默认为0
        """
        ...

    async def unmute(
            self, *,
            target: int,
            member_id: int,
        ) -> Dict[str, Any]:
        """
        指定群解除指定群员禁言（需要有相关限权）

        Args:
            target: 指定群的群号
            member_id: 指定群员QQ号
        """
        ...

    async def kick(
            self, *,
            target: int,
            member_id: int,
            msg: Optional[str]
        ) -> Dict[str, Any]:
        """
        移除指定群成员（需要有相关限权）

        Args:
            target: 指定群的群号
            member_id: 指定群员QQ号
            msg: 信息
        """
        ...

    async def quit(
            self, *,
            target: int
        ) -> Dict[str, Any]:
        """
        使 Bot 退出群聊

        Args:
            target: 指定群的群号
        """
        ...

    async def group_config(
            self, *,
            target: int,
            config: Dict[str, Union[str, bool]]
        ) -> Dict[str, Any]:
        """
        修改群设置（需要有相关限权）

        Args:
            target: 指定群的群号
            config: 群设置（见 https://github.com/mamoe/mirai-api-http#群设置 ）
        """
        ...

    async def get_group_config(
            self, *,
            target: int
        ) -> Dict[str, Any]:
        """
        获取群设置

        Args:
            target: 指定群的群号
        """
        ...

    async def member_info(
            self, *,
            target: int,
            member_id: int,
            info: Dict[str, str]
        ) -> Dict[str, Any]:
        """
        修改群员资料（需要有相关限权）

        Args:
            target: 指定群的群号
            member_id: 群员QQ号
            info: 群员资料（见 https://github.com/mamoe/mirai-api-http#修改群员资料 ）
        """
        ...

    async def get_member_info(
            self, *,
            target: int,
            member_id: int
        ) -> Dict[str, Any]:
        """
        获取群员资料

        Args:
            target: 指定群的群号
            member_id: 群员QQ号
        """
        ...

    async def get_config(
            self
        ) -> Dict[str, Any]:
        """
        获取当前Session的配置信息，注意该配置是Session范围有效
        """
        ...

    async def config(
            self, *,
            cache_size: Optional[int],
            enable_websocket: Optional[bool]
        ) -> Dict[str, Any]:
        """
        设置当前Session的配置信息，注意该配置是Session范围有效

        Args:
            cache_size: 缓存大小
            enable_websocket: 是否开启Websocket
        """
        ...

    async def resp_new_friend(
            self, *,
            event_id: int,
            from_id: int,
            group_id: int,
            operate: Optional[int] = 0,
            message: Optional[str] = ""
        ) -> Dict[str, Any]:
        """
        响应添加好友申请

        Args:
            event_id: 响应申请事件的标识
            from_id: 事件对应申请人 QQ 号
            group_id: 事件对应申请人的群号，可能为0
            operate: 响应的操作类型，0表示同意添加好友，1表示拒绝添加好友，2表示拒绝添加好友并添加黑名单，
                     不再接收该用户的好友申请
            message: 回复的信息
        """
        ...

    async def resp_member_join(
            self, *,
            event_id: int,
            from_id: int,
            group_id: int,
            operate: Optional[int] = 0,
            message: Optional[str] = ""
        ) -> Dict[str, Any]:
        """
        响应用户入群申请（Bot需要有管理员权限）

        Args:
            event_id: 响应申请事件的标识
            from_id: 事件对应申请人QQ号
            group_id: 事件对应申请人的群号，可能为0
            operate: 响应的操作类型，0表示同意入群，1表示拒绝入群，2表示忽略请求，3表示拒绝入群并添加黑名单，
                     不再接收该用户的入群申请，4表示忽略入群并添加黑名单，不再接收该用户的入群申请
            message: 回复的信息
        """
        ...



"""
此模块提供了 Mirai API HTTP 事件接收相关类。
"""

from typing import Any, Dict

from ..bus import EventBus
from ..event import from_payload
from ..logger import Receiver as Logger

__all__ = ['Receiver', 'ReportReceiver']


class Receiver(EventBus):
    async def _handle_event(self, payload: Dict[str, Any]) -> Any:
        Logger.info('Received: %s', str(payload))
        ev = from_payload(payload)
        if not ev:
            return

        event_name = ev.type

        results = list(
            filter(lambda r: r is not None, await self.emit(event_name, ev)))
        # return the first non-none result
        return results[0] if results else None


try:
    from .report import ReportReceiver
except:
    pass



"""
此模块提供了异常类。
"""


class Error(Exception):
    """`aiomirai` 所有异常的基类。"""
    pass


class ApiError(Error, RuntimeError):
    """调用 API 发生错误。"""
    pass


class Unauthenticated(ApiError):
    """未认证。"""
    pass


class ActionFailed(ApiError):
    """
    Mirai API HTTP 已收到 API 请求，但执行失败。
    """
    def __init__(self, code: int, msg: str):
        self.code = code
        self.msg = msg

    def __repr__(self):
        return f'<{self.__class__.__name__}, code={self.code}, msg={self.msg}>'

    def __str__(self):
        return self.__repr__()


class AuthenticateError(ActionFailed):
    """认证相关错误。"""
    pass


class InvalidAuthKey(AuthenticateError):
    """Auth Key错误。"""
    pass


class InvalidBot(AuthenticateError):
    """指定的 Bot 不存在。"""
    pass


class InvalidSession(AuthenticateError):
    """Session 失效或不存在。"""
    pass


class Unverified(AuthenticateError):
    """Session 未认证（未激活）。"""
    pass


class HttpFailed(ApiError):
    """HTTP 请求响应码不是 2xx。"""
    def __init__(self, status_code: int):
        self.status_code = status_code
        """HTTP 响应码。"""

    def __repr__(self):
        return f'<HttpFailed, status_code={self.status_code}>'

    def __str__(self):
        return self.__repr__()


class NetworkError(Error, IOError):
    """网络错误。"""
    pass
